# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.create_fixed_rate_request import CreateFixedRateRequest
from ..types.list_rate_cards_response import ListRateCardsResponse
from ..types.rate_card_resource import RateCardResource
from .raw_client import AsyncRawRateCardsClient, RawRateCardsClient
from .types.create_rate_card_request_billing_interval import CreateRateCardRequestBillingInterval
from .types.create_rate_card_request_usage_based_rates_item import CreateRateCardRequestUsageBasedRatesItem

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RateCardsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawRateCardsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawRateCardsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawRateCardsClient
        """
        return self._raw_client

    def list_rate_cards(
        self, *, limit: typing.Optional[int] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> ListRateCardsResponse:
        """
        Parameters
        ----------
        limit : typing.Optional[int]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ListRateCardsResponse
            Successful Response

        Examples
        --------
        from lark import Lark

        client = Lark(
            api_key="YOUR_API_KEY",
        )
        client.rate_cards.list_rate_cards(
            limit=1,
        )
        """
        _response = self._raw_client.list_rate_cards(limit=limit, request_options=request_options)
        return _response.data

    def create_rate_card(
        self,
        *,
        name: str,
        billing_interval: CreateRateCardRequestBillingInterval,
        description: typing.Optional[str] = OMIT,
        fixed_rates: typing.Optional[typing.Sequence[CreateFixedRateRequest]] = OMIT,
        usage_based_rates: typing.Optional[typing.Sequence[CreateRateCardRequestUsageBasedRatesItem]] = OMIT,
        metadata: typing.Optional[typing.Dict[str, str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> RateCardResource:
        """
        Parameters
        ----------
        name : str
            The name of the rate card displayed to the customer.

        billing_interval : CreateRateCardRequestBillingInterval
            How often the customer will be billed for this rate card.

        description : typing.Optional[str]
            The description of the rate card displayed to the customer.

        fixed_rates : typing.Optional[typing.Sequence[CreateFixedRateRequest]]
            The fixed rates of the rate card. These are billed at the start of each billing cycle.

        usage_based_rates : typing.Optional[typing.Sequence[CreateRateCardRequestUsageBasedRatesItem]]
            The usage based rates of the rate card. These are billed at the end of each billing cycle.

        metadata : typing.Optional[typing.Dict[str, str]]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RateCardResource
            Successful Response

        Examples
        --------
        from lark import Amount, CreateFixedRateRequest, Lark, Price_Flat
        from lark.rate_cards import CreateRateCardRequestUsageBasedRatesItem_Simple

        client = Lark(
            api_key="YOUR_API_KEY",
        )
        client.rate_cards.create_rate_card(
            name="Pro Plan",
            description="For production applications with moderate usage.",
            billing_interval="monthly",
            fixed_rates=[
                CreateFixedRateRequest(
                    name="Base Rate",
                    price=Price_Flat(
                        amount=Amount(
                            value="2500",
                            currency_code="usd",
                        ),
                    ),
                )
            ],
            usage_based_rates=[
                CreateRateCardRequestUsageBasedRatesItem_Simple(
                    name="name",
                    price=Price_Flat(
                        amount=Amount(
                            value="2500",
                            currency_code="usd",
                        ),
                    ),
                    pricing_metric_id="pricing_metric_id",
                )
            ],
            metadata={"key": "value"},
        )
        """
        _response = self._raw_client.create_rate_card(
            name=name,
            billing_interval=billing_interval,
            description=description,
            fixed_rates=fixed_rates,
            usage_based_rates=usage_based_rates,
            metadata=metadata,
            request_options=request_options,
        )
        return _response.data

    def get_rate_card(
        self, rate_card_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> RateCardResource:
        """
        Parameters
        ----------
        rate_card_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RateCardResource
            Successful Response

        Examples
        --------
        from lark import Lark

        client = Lark(
            api_key="YOUR_API_KEY",
        )
        client.rate_cards.get_rate_card(
            rate_card_id="rate_card_id",
        )
        """
        _response = self._raw_client.get_rate_card(rate_card_id, request_options=request_options)
        return _response.data


class AsyncRateCardsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawRateCardsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawRateCardsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawRateCardsClient
        """
        return self._raw_client

    async def list_rate_cards(
        self, *, limit: typing.Optional[int] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> ListRateCardsResponse:
        """
        Parameters
        ----------
        limit : typing.Optional[int]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ListRateCardsResponse
            Successful Response

        Examples
        --------
        import asyncio

        from lark import AsyncLark

        client = AsyncLark(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.rate_cards.list_rate_cards(
                limit=1,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.list_rate_cards(limit=limit, request_options=request_options)
        return _response.data

    async def create_rate_card(
        self,
        *,
        name: str,
        billing_interval: CreateRateCardRequestBillingInterval,
        description: typing.Optional[str] = OMIT,
        fixed_rates: typing.Optional[typing.Sequence[CreateFixedRateRequest]] = OMIT,
        usage_based_rates: typing.Optional[typing.Sequence[CreateRateCardRequestUsageBasedRatesItem]] = OMIT,
        metadata: typing.Optional[typing.Dict[str, str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> RateCardResource:
        """
        Parameters
        ----------
        name : str
            The name of the rate card displayed to the customer.

        billing_interval : CreateRateCardRequestBillingInterval
            How often the customer will be billed for this rate card.

        description : typing.Optional[str]
            The description of the rate card displayed to the customer.

        fixed_rates : typing.Optional[typing.Sequence[CreateFixedRateRequest]]
            The fixed rates of the rate card. These are billed at the start of each billing cycle.

        usage_based_rates : typing.Optional[typing.Sequence[CreateRateCardRequestUsageBasedRatesItem]]
            The usage based rates of the rate card. These are billed at the end of each billing cycle.

        metadata : typing.Optional[typing.Dict[str, str]]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RateCardResource
            Successful Response

        Examples
        --------
        import asyncio

        from lark import Amount, AsyncLark, CreateFixedRateRequest, Price_Flat
        from lark.rate_cards import CreateRateCardRequestUsageBasedRatesItem_Simple

        client = AsyncLark(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.rate_cards.create_rate_card(
                name="Pro Plan",
                description="For production applications with moderate usage.",
                billing_interval="monthly",
                fixed_rates=[
                    CreateFixedRateRequest(
                        name="Base Rate",
                        price=Price_Flat(
                            amount=Amount(
                                value="2500",
                                currency_code="usd",
                            ),
                        ),
                    )
                ],
                usage_based_rates=[
                    CreateRateCardRequestUsageBasedRatesItem_Simple(
                        name="name",
                        price=Price_Flat(
                            amount=Amount(
                                value="2500",
                                currency_code="usd",
                            ),
                        ),
                        pricing_metric_id="pricing_metric_id",
                    )
                ],
                metadata={"key": "value"},
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create_rate_card(
            name=name,
            billing_interval=billing_interval,
            description=description,
            fixed_rates=fixed_rates,
            usage_based_rates=usage_based_rates,
            metadata=metadata,
            request_options=request_options,
        )
        return _response.data

    async def get_rate_card(
        self, rate_card_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> RateCardResource:
        """
        Parameters
        ----------
        rate_card_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RateCardResource
            Successful Response

        Examples
        --------
        import asyncio

        from lark import AsyncLark

        client = AsyncLark(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.rate_cards.get_rate_card(
                rate_card_id="rate_card_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_rate_card(rate_card_id, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.list_subscriptions_response import ListSubscriptionsResponse
from ..types.subscription_resource import SubscriptionResource
from .raw_client import AsyncRawSubscriptionsClient, RawSubscriptionsClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class SubscriptionsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawSubscriptionsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawSubscriptionsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawSubscriptionsClient
        """
        return self._raw_client

    def list_subscriptions(
        self,
        *,
        limit: typing.Optional[int] = None,
        subject_id: typing.Optional[str] = None,
        rate_card_id: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ListSubscriptionsResponse:
        """
        Parameters
        ----------
        limit : typing.Optional[int]

        subject_id : typing.Optional[str]
            The ID of the subject to list subscriptions for. Cannot be used with rate_card_id.

        rate_card_id : typing.Optional[str]
            The ID of the rate card to list subscriptions for. Cannot be used with subject_id.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ListSubscriptionsResponse
            Successful Response

        Examples
        --------
        from lark import Lark

        client = Lark(
            api_key="YOUR_API_KEY",
        )
        client.subscriptions.list_subscriptions(
            limit=1,
            subject_id="subject_id",
            rate_card_id="rate_card_id",
        )
        """
        _response = self._raw_client.list_subscriptions(
            limit=limit, subject_id=subject_id, rate_card_id=rate_card_id, request_options=request_options
        )
        return _response.data

    def create_subscription(
        self,
        *,
        rate_card_id: str,
        subject_id: str,
        metadata: typing.Optional[typing.Dict[str, str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SubscriptionResource:
        """
        Parameters
        ----------
        rate_card_id : str
            The ID of the rate card to use for the subscription.

        subject_id : str
            The ID of the subject to create the subscription for.

        metadata : typing.Optional[typing.Dict[str, str]]
            Additional metadata about the subscription. You may use this to store any custom data about the subscription.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SubscriptionResource
            Successful Response

        Examples
        --------
        from lark import Lark

        client = Lark(
            api_key="YOUR_API_KEY",
        )
        client.subscriptions.create_subscription(
            rate_card_id="rc_AJWMxR81jxoRlli6p13uf3JB",
            subject_id="subj_VyX6Q96h5avMho8O7QWlKeXE",
        )
        """
        _response = self._raw_client.create_subscription(
            rate_card_id=rate_card_id, subject_id=subject_id, metadata=metadata, request_options=request_options
        )
        return _response.data

    def get_subscription(
        self, subscription_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> SubscriptionResource:
        """
        Parameters
        ----------
        subscription_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SubscriptionResource
            Successful Response

        Examples
        --------
        from lark import Lark

        client = Lark(
            api_key="YOUR_API_KEY",
        )
        client.subscriptions.get_subscription(
            subscription_id="subscription_id",
        )
        """
        _response = self._raw_client.get_subscription(subscription_id, request_options=request_options)
        return _response.data

    def cancel_subscription(
        self,
        subscription_id: str,
        *,
        reason: typing.Optional[str] = OMIT,
        cancel_at_end_of_cycle: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SubscriptionResource:
        """
        Parameters
        ----------
        subscription_id : str

        reason : typing.Optional[str]
            The reason for cancelling the subscription.

        cancel_at_end_of_cycle : typing.Optional[bool]
            Whether to cancel the subscription at end of cycle.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SubscriptionResource
            Successful Response

        Examples
        --------
        from lark import Lark

        client = Lark(
            api_key="YOUR_API_KEY",
        )
        client.subscriptions.cancel_subscription(
            subscription_id="subscription_id",
        )
        """
        _response = self._raw_client.cancel_subscription(
            subscription_id,
            reason=reason,
            cancel_at_end_of_cycle=cancel_at_end_of_cycle,
            request_options=request_options,
        )
        return _response.data

    def change_subscription_rate_card(
        self, subscription_id: str, *, rate_card_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> SubscriptionResource:
        """
        Parameters
        ----------
        subscription_id : str

        rate_card_id : str
            The ID of the rate card to change the subscription to.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SubscriptionResource
            Successful Response

        Examples
        --------
        from lark import Lark

        client = Lark(
            api_key="YOUR_API_KEY",
        )
        client.subscriptions.change_subscription_rate_card(
            subscription_id="subscription_id",
            rate_card_id="rc_jQK2n0wutCj6bBcAIrL6o07g",
        )
        """
        _response = self._raw_client.change_subscription_rate_card(
            subscription_id, rate_card_id=rate_card_id, request_options=request_options
        )
        return _response.data


class AsyncSubscriptionsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawSubscriptionsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawSubscriptionsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawSubscriptionsClient
        """
        return self._raw_client

    async def list_subscriptions(
        self,
        *,
        limit: typing.Optional[int] = None,
        subject_id: typing.Optional[str] = None,
        rate_card_id: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ListSubscriptionsResponse:
        """
        Parameters
        ----------
        limit : typing.Optional[int]

        subject_id : typing.Optional[str]
            The ID of the subject to list subscriptions for. Cannot be used with rate_card_id.

        rate_card_id : typing.Optional[str]
            The ID of the rate card to list subscriptions for. Cannot be used with subject_id.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ListSubscriptionsResponse
            Successful Response

        Examples
        --------
        import asyncio

        from lark import AsyncLark

        client = AsyncLark(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.subscriptions.list_subscriptions(
                limit=1,
                subject_id="subject_id",
                rate_card_id="rate_card_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.list_subscriptions(
            limit=limit, subject_id=subject_id, rate_card_id=rate_card_id, request_options=request_options
        )
        return _response.data

    async def create_subscription(
        self,
        *,
        rate_card_id: str,
        subject_id: str,
        metadata: typing.Optional[typing.Dict[str, str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SubscriptionResource:
        """
        Parameters
        ----------
        rate_card_id : str
            The ID of the rate card to use for the subscription.

        subject_id : str
            The ID of the subject to create the subscription for.

        metadata : typing.Optional[typing.Dict[str, str]]
            Additional metadata about the subscription. You may use this to store any custom data about the subscription.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SubscriptionResource
            Successful Response

        Examples
        --------
        import asyncio

        from lark import AsyncLark

        client = AsyncLark(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.subscriptions.create_subscription(
                rate_card_id="rc_AJWMxR81jxoRlli6p13uf3JB",
                subject_id="subj_VyX6Q96h5avMho8O7QWlKeXE",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create_subscription(
            rate_card_id=rate_card_id, subject_id=subject_id, metadata=metadata, request_options=request_options
        )
        return _response.data

    async def get_subscription(
        self, subscription_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> SubscriptionResource:
        """
        Parameters
        ----------
        subscription_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SubscriptionResource
            Successful Response

        Examples
        --------
        import asyncio

        from lark import AsyncLark

        client = AsyncLark(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.subscriptions.get_subscription(
                subscription_id="subscription_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_subscription(subscription_id, request_options=request_options)
        return _response.data

    async def cancel_subscription(
        self,
        subscription_id: str,
        *,
        reason: typing.Optional[str] = OMIT,
        cancel_at_end_of_cycle: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SubscriptionResource:
        """
        Parameters
        ----------
        subscription_id : str

        reason : typing.Optional[str]
            The reason for cancelling the subscription.

        cancel_at_end_of_cycle : typing.Optional[bool]
            Whether to cancel the subscription at end of cycle.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SubscriptionResource
            Successful Response

        Examples
        --------
        import asyncio

        from lark import AsyncLark

        client = AsyncLark(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.subscriptions.cancel_subscription(
                subscription_id="subscription_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.cancel_subscription(
            subscription_id,
            reason=reason,
            cancel_at_end_of_cycle=cancel_at_end_of_cycle,
            request_options=request_options,
        )
        return _response.data

    async def change_subscription_rate_card(
        self, subscription_id: str, *, rate_card_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> SubscriptionResource:
        """
        Parameters
        ----------
        subscription_id : str

        rate_card_id : str
            The ID of the rate card to change the subscription to.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SubscriptionResource
            Successful Response

        Examples
        --------
        import asyncio

        from lark import AsyncLark

        client = AsyncLark(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.subscriptions.change_subscription_rate_card(
                subscription_id="subscription_id",
                rate_card_id="rc_jQK2n0wutCj6bBcAIrL6o07g",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.change_subscription_rate_card(
            subscription_id, rate_card_id=rate_card_id, request_options=request_options
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..errors.unprocessable_entity_error import UnprocessableEntityError
from ..types.http_validation_error import HttpValidationError
from ..types.list_subscriptions_response import ListSubscriptionsResponse
from ..types.subscription_resource import SubscriptionResource

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawSubscriptionsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list_subscriptions(
        self,
        *,
        limit: typing.Optional[int] = None,
        subject_id: typing.Optional[str] = None,
        rate_card_id: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[ListSubscriptionsResponse]:
        """
        Parameters
        ----------
        limit : typing.Optional[int]

        subject_id : typing.Optional[str]
            The ID of the subject to list subscriptions for. Cannot be used with rate_card_id.

        rate_card_id : typing.Optional[str]
            The ID of the rate card to list subscriptions for. Cannot be used with subject_id.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ListSubscriptionsResponse]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "subscriptions",
            method="GET",
            params={
                "limit": limit,
                "subject_id": subject_id,
                "rate_card_id": rate_card_id,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ListSubscriptionsResponse,
                    parse_obj_as(
                        type_=ListSubscriptionsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create_subscription(
        self,
        *,
        rate_card_id: str,
        subject_id: str,
        metadata: typing.Optional[typing.Dict[str, str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[SubscriptionResource]:
        """
        Parameters
        ----------
        rate_card_id : str
            The ID of the rate card to use for the subscription.

        subject_id : str
            The ID of the subject to create the subscription for.

        metadata : typing.Optional[typing.Dict[str, str]]
            Additional metadata about the subscription. You may use this to store any custom data about the subscription.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[SubscriptionResource]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "subscriptions",
            method="POST",
            json={
                "rate_card_id": rate_card_id,
                "subject_id": subject_id,
                "metadata": metadata,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SubscriptionResource,
                    parse_obj_as(
                        type_=SubscriptionResource,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_subscription(
        self, subscription_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[SubscriptionResource]:
        """
        Parameters
        ----------
        subscription_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[SubscriptionResource]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"subscriptions/{jsonable_encoder(subscription_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SubscriptionResource,
                    parse_obj_as(
                        type_=SubscriptionResource,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def cancel_subscription(
        self,
        subscription_id: str,
        *,
        reason: typing.Optional[str] = OMIT,
        cancel_at_end_of_cycle: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[SubscriptionResource]:
        """
        Parameters
        ----------
        subscription_id : str

        reason : typing.Optional[str]
            The reason for cancelling the subscription.

        cancel_at_end_of_cycle : typing.Optional[bool]
            Whether to cancel the subscription at end of cycle.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[SubscriptionResource]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"subscriptions/{jsonable_encoder(subscription_id)}/cancel",
            method="POST",
            json={
                "reason": reason,
                "cancel_at_end_of_cycle": cancel_at_end_of_cycle,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SubscriptionResource,
                    parse_obj_as(
                        type_=SubscriptionResource,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def change_subscription_rate_card(
        self, subscription_id: str, *, rate_card_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[SubscriptionResource]:
        """
        Parameters
        ----------
        subscription_id : str

        rate_card_id : str
            The ID of the rate card to change the subscription to.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[SubscriptionResource]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"subscriptions/{jsonable_encoder(subscription_id)}/change-rate-card",
            method="POST",
            json={
                "rate_card_id": rate_card_id,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SubscriptionResource,
                    parse_obj_as(
                        type_=SubscriptionResource,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawSubscriptionsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list_subscriptions(
        self,
        *,
        limit: typing.Optional[int] = None,
        subject_id: typing.Optional[str] = None,
        rate_card_id: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[ListSubscriptionsResponse]:
        """
        Parameters
        ----------
        limit : typing.Optional[int]

        subject_id : typing.Optional[str]
            The ID of the subject to list subscriptions for. Cannot be used with rate_card_id.

        rate_card_id : typing.Optional[str]
            The ID of the rate card to list subscriptions for. Cannot be used with subject_id.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ListSubscriptionsResponse]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "subscriptions",
            method="GET",
            params={
                "limit": limit,
                "subject_id": subject_id,
                "rate_card_id": rate_card_id,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ListSubscriptionsResponse,
                    parse_obj_as(
                        type_=ListSubscriptionsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create_subscription(
        self,
        *,
        rate_card_id: str,
        subject_id: str,
        metadata: typing.Optional[typing.Dict[str, str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[SubscriptionResource]:
        """
        Parameters
        ----------
        rate_card_id : str
            The ID of the rate card to use for the subscription.

        subject_id : str
            The ID of the subject to create the subscription for.

        metadata : typing.Optional[typing.Dict[str, str]]
            Additional metadata about the subscription. You may use this to store any custom data about the subscription.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[SubscriptionResource]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "subscriptions",
            method="POST",
            json={
                "rate_card_id": rate_card_id,
                "subject_id": subject_id,
                "metadata": metadata,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SubscriptionResource,
                    parse_obj_as(
                        type_=SubscriptionResource,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_subscription(
        self, subscription_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[SubscriptionResource]:
        """
        Parameters
        ----------
        subscription_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[SubscriptionResource]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"subscriptions/{jsonable_encoder(subscription_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SubscriptionResource,
                    parse_obj_as(
                        type_=SubscriptionResource,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def cancel_subscription(
        self,
        subscription_id: str,
        *,
        reason: typing.Optional[str] = OMIT,
        cancel_at_end_of_cycle: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[SubscriptionResource]:
        """
        Parameters
        ----------
        subscription_id : str

        reason : typing.Optional[str]
            The reason for cancelling the subscription.

        cancel_at_end_of_cycle : typing.Optional[bool]
            Whether to cancel the subscription at end of cycle.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[SubscriptionResource]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"subscriptions/{jsonable_encoder(subscription_id)}/cancel",
            method="POST",
            json={
                "reason": reason,
                "cancel_at_end_of_cycle": cancel_at_end_of_cycle,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SubscriptionResource,
                    parse_obj_as(
                        type_=SubscriptionResource,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def change_subscription_rate_card(
        self, subscription_id: str, *, rate_card_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[SubscriptionResource]:
        """
        Parameters
        ----------
        subscription_id : str

        rate_card_id : str
            The ID of the rate card to change the subscription to.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[SubscriptionResource]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"subscriptions/{jsonable_encoder(subscription_id)}/change-rate-card",
            method="POST",
            json={
                "rate_card_id": rate_card_id,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SubscriptionResource,
                    parse_obj_as(
                        type_=SubscriptionResource,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)
